from PIL import Image, ImageDraw, ImageFont
import imageio
import os

def text_to_gif(text, output_filename="output.gif", font_size=50, frame_count=10, frame_duration=100):
    """
    Creates an animated GIF from the given text.
    
    :param text: The text to display in the GIF.
    :param output_filename: Name of the output GIF file.
    :param font_size: Size of the font used in the image.
    :param frame_count: Number of frames in the animation.
    :param frame_duration: Duration of each frame in milliseconds.
    """
    # Create a temporary directory for frames
    temp_dir = "temp_frames"
    os.makedirs(temp_dir, exist_ok=True)
    
    # Load font (ensure you have Arial or specify a font path)
    try:
        font = ImageFont.truetype("arial.ttf", font_size)
    except:
        font = ImageFont.load_default()
    
    # Define image size
    img_width, img_height = 400, 200
    
    frames = []
    frame_paths = []
    for i in range(frame_count):
        img = Image.new("RGB", (img_width, img_height), "white")
        draw = ImageDraw.Draw(img)
        text_position = (20, 80 + int(10 * (i - frame_count / 2)))  # Slight movement
        draw.text(text_position, text, fill="black", font=font)
        
        frame_path = os.path.join(temp_dir, f"frame_{i}.png")
        img.save(frame_path)
        frame_paths.append(frame_path)
        frames.append(imageio.imread(frame_path))
    
    # Create GIF
    imageio.mimsave(output_filename, frames, duration=frame_duration / 1000)
    
    # Cleanup
    for frame_path in frame_paths:
        if os.path.exists(frame_path):
            os.remove(frame_path)
    os.rmdir(temp_dir)
    
    print(f"GIF saved as {output_filename}")

# Example Usage
text_to_gif("Hello, World!", "animated_text.gif")
